@@ -21,6 +21,9 @@
 package cmd
 
 import (
+	"bytes"
+	"crypto/md5"
+	"encoding/hex"
 	"fmt"
 	"io"
 	"math"
@@ -28,7 +31,6 @@ import (
 	"sort"
 	"strings"
 
-	"github.com/cespare/xxhash"
 	"github.com/shenwei356/bio/seq"
 	"github.com/shenwei356/bio/seqio/fastx"
 	"github.com/shenwei356/xopen"
@@ -38,7 +40,7 @@ import (
 // fx2tabCmd represents the fx2tab command
 var fx2tabCmd = &cobra.Command{
 	Use:   "fx2tab",
-	Short: "convert FASTA/Q to tabular format (with length/GC content/GC skew)",
+	Short: "convert FASTA/Q to tabular format (and length, GC content, average quality...)",
 	Long: `convert FASTA/Q to tabular format, and provide various information,
 like sequence length, GC content/GC skew.
 
@@ -63,6 +65,7 @@ Attention:
 		printGC := getFlagBool(cmd, "gc")
 		printGCSkew := getFlagBool(cmd, "gc-skew")
 		baseContents := getFlagStringSlice(cmd, "base-content")
+		baseCounts := getFlagStringSlice(cmd, "base-count")
 		caseSensitive := getFlagBool(cmd, "case-sensitive")
 		onlyName := getFlagBool(cmd, "name")
 		printTitle := getFlagBool(cmd, "header-line")
@@ -70,6 +73,7 @@ Attention:
 		printAvgQual := getFlagBool(cmd, "avg-qual")
 		qBase := getFlagPositiveInt(cmd, "qual-ascii-base")
 		printSeqHash := getFlagBool(cmd, "seq-hash")
+		noQual := getFlagBool(cmd, "no-qual")
 
 		outfh, err := xopen.Wopen(outFile)
 		checkError(err)
@@ -84,9 +88,18 @@ Attention:
 				}
 			} else {
 				if onlyID {
-					outfh.WriteString("#id\tseq\tqual")
+					if noQual {
+						outfh.WriteString("#id\tseq")
+					} else {
+						outfh.WriteString("#id\tseq\tqual")
+					}
 				} else {
-					outfh.WriteString("#name\tseq\tqual")
+					if noQual {
+						outfh.WriteString("#name\tseq")
+					} else {
+						outfh.WriteString("#name\tseq\tqual")
+					}
+
 				}
 			}
 			if printLength {
@@ -98,6 +111,11 @@ Attention:
 			if printGCSkew {
 				outfh.WriteString("\tGC-Skew")
 			}
+			if len(baseCounts) > 0 {
+				for _, bc := range baseCounts {
+					outfh.WriteString(fmt.Sprintf("\t%s", bc))
+				}
+			}
 			if len(baseContents) > 0 {
 				for _, bc := range baseContents {
 					outfh.WriteString(fmt.Sprintf("\t%s", bc))
@@ -120,6 +138,7 @@ Attention:
 		var g, c float64
 		var record *fastx.Record
 		var fastxReader *fastx.Reader
+		var sum [md5.Size]byte
 		for _, file := range files {
 			fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
 			checkError(err)
@@ -142,11 +161,15 @@ Attention:
 				} else {
 					//outfh.WriteString(fmt.Sprintf("%s\t%s\t%s", name,
 					//	record.Seq.Seq, record.Seq.Qual))
-					outfh.WriteString(fmt.Sprintf("%s\t", name))
-					outfh.Write(record.Seq.Seq)
-					outfh.WriteString("\t")
-					outfh.Write(record.Seq.Qual)
 
+					// outfh.WriteString(fmt.Sprintf("%s\t", name))
+					outfh.Write(name)
+					outfh.Write(_tab)
+					outfh.Write(record.Seq.Seq)
+					if !noQual {
+						outfh.Write(_tab)
+						outfh.Write(record.Seq.Qual)
+					}
 				}
 
 				if printLength {
@@ -164,6 +187,16 @@ Attention:
 					outfh.WriteString(fmt.Sprintf("\t%.2f", (g-c)/(g+c)*100))
 				}
 
+				if len(baseCounts) > 0 {
+					for _, bc := range baseCounts {
+						if caseSensitive {
+							outfh.WriteString(fmt.Sprintf("\t%d", record.Seq.BaseCountCaseSensitive(bc)))
+						} else {
+							outfh.WriteString(fmt.Sprintf("\t%d", record.Seq.BaseCount(bc)))
+						}
+					}
+				}
+
 				if len(baseContents) > 0 {
 					for _, bc := range baseContents {
 						if caseSensitive {
@@ -183,10 +216,18 @@ Attention:
 				}
 
 				if printSeqHash {
-					outfh.WriteString(fmt.Sprintf("\t%d", xxhash.Sum64(record.Seq.Seq)))
+					if caseSensitive {
+						sum = md5.Sum(record.Seq.Seq)
+						outfh.WriteString(fmt.Sprintf("\t%s", hex.EncodeToString(sum[:])))
+					} else {
+						sum = md5.Sum(bytes.ToLower(record.Seq.Seq))
+						outfh.WriteString(fmt.Sprintf("\t%s", hex.EncodeToString(sum[:])))
+					}
+
 				}
 
-				outfh.WriteString("\n")
+				// outfh.WriteString("\n")
+				outfh.Write(_mark_newline)
 			}
 		}
 	},
@@ -199,14 +240,16 @@ func init() {
 	fx2tabCmd.Flags().BoolP("gc", "g", false, "print GC content")
 	fx2tabCmd.Flags().BoolP("gc-skew", "G", false, "print GC-Skew")
 	fx2tabCmd.Flags().StringSliceP("base-content", "B", []string{}, "print base content. (case ignored, multiple values supported) e.g. -B AT -B N")
-	fx2tabCmd.Flags().BoolP("case-sensitive", "I", false, "calculate case sensitive base content")
+	fx2tabCmd.Flags().StringSliceP("base-count", "C", []string{}, "print base count. (case ignored, multiple values supported) e.g. -C AT -C N")
+	fx2tabCmd.Flags().BoolP("case-sensitive", "I", false, "calculate case sensitive base content/sequence hash")
 	fx2tabCmd.Flags().BoolP("only-id", "i", false, "print ID instead of full head")
 	fx2tabCmd.Flags().BoolP("name", "n", false, "only print names (no sequences and qualities)")
 	fx2tabCmd.Flags().BoolP("header-line", "H", false, "print header line")
 	fx2tabCmd.Flags().BoolP("alphabet", "a", false, "print alphabet letters")
 	fx2tabCmd.Flags().BoolP("avg-qual", "q", false, "print average quality of a read")
 	fx2tabCmd.Flags().IntP("qual-ascii-base", "b", 33, "ASCII BASE, 33 for Phred+33")
-	fx2tabCmd.Flags().BoolP("seq-hash", "s", false, "print hash of sequence (case sensitive)")
+	fx2tabCmd.Flags().BoolP("seq-hash", "s", false, "print hash (MD5) of sequence")
+	fx2tabCmd.Flags().BoolP("no-qual", "Q", false, "only output two column even for FASTQ file")
 
 }
 
@@ -236,3 +279,5 @@ func avgQual(s *seq.Seq, base int) float64 {
 	}
 	return -10 * math.Log10(sum/float64(len(s.QualValue)))
 }
+
+var _tab = []byte{'\t'}
