@@ -216,6 +216,9 @@ Secondly, seqkit shuffles sequence IDs and extract sequences by FASTA index.
 			sequence := subseqByFaixNotCleaned(faidx, chr, r, 1, -1)
 			outfh.Write([]byte(fmt.Sprintf(">%s\n", chr)))
 			outfh.Write(sequence)
+			if len(sequence) > 0 && sequence[len(sequence)-1] != '\n' {
+				outfh.WriteString("\n")
+			}
 		}
 
 		if (isStdin(file) || !isPlainFile(file)) && !keepTemp {
